from ymci.ext.db import Table
from sqlalchemy import Column, Integer, ForeignKeyConstraint, Boolean
from sqlalchemy.orm import relationship, backref


class OAuthProjectConfig(Table):
    __tablename__ = 'ymci_ext_oauth_config'
    __table_args__ = (
        ForeignKeyConstraint(['project_id'], ['project.project_id']),
    )
    project_id = Column(Integer, primary_key=True, nullable=False)
    public_read = Column(Boolean)
    public_build = Column(Boolean)

    columns_list = ['public_read', 'public_build']

    project = relationship('Project', backref=backref(
        'oauth_config', uselist=False, cascade='all'))



from ymci.ext.form import Form
from .db import OAuthProjectConfig


class OAuthConfigForm(Form):
    class Meta(object):
        model = OAuthProjectConfig
        only = OAuthProjectConfig.columns_list



from ymci.ext.routes import url, Route
from tornado.auth import GoogleOAuth2Mixin
from tornado.gen import coroutine
from tornado.escape import json_encode, json_decode
from tornado.web import HTTPError
from tornado.httpclient import AsyncHTTPClient
from ymci import server


@url(r'/_oauth2callback')
class GoogleOAuth2LoginHandler(Route, GoogleOAuth2Mixin):
    google_info_url = 'https://www.googleapis.com/oauth2/v2/userinfo'

    @property
    def oauth_url(self):
        return '%s://%s/_oauth2callback' % (
            self.application.settings.protocol, self.request.host
        )

    @coroutine
    def get(self):
        if self.get_argument('code', False):
            user = yield self.get_authenticated_user(
                redirect_uri=self.oauth_url,
                code=self.get_argument('code'))
            self.log.debug('Got user %s' % user)
            if not user and 'token' not in user:
                self.clear_all_cookies()
                self.set_flash_message('danger', 'Unknown user')
                raise HTTPError(403)

            if not self.settings['google_oauth'].get('domain', ''):
                self.set_secure_cookie('user', json_encode(user))
                self.set_flash_message('success', 'Login successful')
                return self.redirect('/')

            access_token = str(user['access_token'])
            response = yield AsyncHTTPClient().fetch(
                self.google_info_url,
                headers={
                    'Authorization': 'Bearer %s' % access_token
                })

            self.log.debug('Got userinfo %s' % response.body)
            if response.code != 200:
                self.set_flash_message('danger', 'Error contacting oauth api')
                raise HTTPError(403)

            body = json_decode(response.body)
            email = body['email']
            if not email.endswith(
                    '@%s' % self.settings['google_oauth']['domain']):
                self.set_flash_message('danger', '%s is not inlol domain %s' % (
                    email, self.settings['google_oauth']['domain']
                ))
                raise HTTPError(403)

            self.set_secure_cookie('user', email, expires_days=1)
            self.set_flash_message(
                'success', 'Login successful for %s' % email)
            return self.redirect('/')

        yield self.authorize_redirect(
            redirect_uri=self.oauth_url,
            client_id=self.settings['google_oauth']['key'],
            scope=['profile', 'email'],
            response_type='code',
            extra_params={
                'approval_prompt': 'auto'
            })


@url(r'/google_auth/logout')
class GoogleOAuth2Logout(Route):
    def get(self):
        self.clear_cookie("user")
        self.set_flash_message('success', 'Logout successful')
        self.redirect("/")


server.components.project_unauth.login = {
    'route': 'GoogleOAuth2LoginHandler',
    'label': 'Login'
}

server.components.project_auth.logout = {
    'route': 'GoogleOAuth2Logout',
    'label': 'Logout'
}



from tornado.web import HTTPError
from ymci.ext.hooks import PrepareHook
from ymci.model import Project, Build
from ymci import server


oauth = server.conf.get('google_oauth')
if oauth:
    server.settings['google_oauth'] = oauth


class OAuthHook(PrepareHook):
    @property
    def active(self):
        return oauth

    def prepare(self, route):
        name = route.__class__.__name__
        current_user = route.get_secure_cookie("user")
        if current_user or not name.startswith('Project') or name in (
                'ProjectList',
        ):
            return
        if not current_user and name == 'ProjectAdd':
            return route.redirect(
                route.reverse_url('GoogleOAuth2LoginHandler'))

        project = None
        if 'project_id' in route.path_kwargs:
            project = self.db.query(Project).get(
                route.path_kwargs['project_id'])
        # if 'build_id' in route.path_kwargs:
        #     project = self.db.query(Build).get(
        #         route.path_kwargs['build_id']).project

        if not project:
            return

        if getattr(project.oauth_config, 'public_read', False) and name in (
                'ProjectView', 'ProjectLog', 'ProjectBuildLog'
        ):
            return

        if getattr(project.oauth_config, 'public_build', False) and name in (
                'ProjectBuild', 'ProjectBuildStop'
        ):
            return

        return route.redirect(
            route.reverse_url('GoogleOAuth2LoginHandler'))



from setuptools import setup, find_packages

setup(
    name='ymci-ext-oauth',
    version='1.1',
    description='OAuth plugin for ymci',
    author='Florian Mounier, Kozea',
    author_email='florian.mounier@kozea.fr',

    platforms=['Any'],

    scripts=[],

    provides=['ymci_ext_oauth'],
    packages=find_packages(),
    package_data={'ymci_ext_oauth': ['templates/*', 'static/*']},

    entry_points={
        'ymci.ext': [
            'ymci-ext-oauth = ymci_ext_oauth'
        ],
        'ymci.ext.db.Table': [
            'oauth_table = ymci_ext_oauth.db:OAuthProjectConfig'
        ],
        'ymci.ext.form.Form': [
            'form = ymci_ext_oauth.form:OAuthConfigForm'
        ],
        'ymci.ext.routes.Route': [
            'result_chart = ymci_ext_oauth.routes:GoogleOAuth2LoginHandler'
        ],
        'ymci.ext.hooks.PrepareHook': [
            'oauth_hook = ymci_ext_oauth.hooks:OAuthHook'
        ]
    }
)
